@@ -21,21 +21,23 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#if defined(HAVE_LIBCURL)
-#include "yyjson.h"
 
 #include <string.h>
 
 #include "core/siril.h"
-#include "core/siril_networking.h"
 #include "core/proto.h"
-#include "core/processing.h"
 #include "core/siril_log.h"
+#include "core/siril_app_dirs.h"
 #include "gui/utils.h"
-#include "gui/message_dialog.h"
-#include "gui/progress_and_log.h"
 #include "core/siril_update.h"
 
+#if defined(HAVE_LIBCURL)
+#include "yyjson.h"
+#include "core/siril_networking.h"
+#include "core/processing.h"
+#include "gui/message_dialog.h"
+#include "gui/progress_and_log.h"
+#include "gui/photometric_cc.h"
 
 #define SIRIL_DOMAIN "https://siril.org/"
 #define SIRIL_VERSIONS SIRIL_DOMAIN"siril_versions.json"
@@ -43,6 +45,175 @@
 #define GITLAB_URL "https://gitlab.com/free-astro/siril/raw"
 #define BRANCH "master"
 #define SIRIL_NOTIFICATIONS "notifications/siril_notifications.json"
+#define SPCC_MIRRORS "notifications/spcc_mirrors.json"
+
+#endif
+
+extern gchar** spcc_mirrors;
+
+// ============================================================================
+// UTILITY FUNCTIONS - Independent of libcurl
+// These functions work with version numbers and strings, no network required
+// ============================================================================
+
+static void remove_alpha(gchar *str, gboolean *is_rc, gboolean *is_beta) {
+	unsigned long i = 0;
+	unsigned long j = 0;
+	char c;
+
+	if (g_str_has_prefix(str, "beta")) {
+		*is_rc = FALSE;
+		*is_beta = TRUE;
+	} else if (g_str_has_prefix(str, "rc")) {
+		*is_rc = TRUE;
+		*is_beta = FALSE;
+	} else {
+		*is_rc = FALSE;
+		*is_beta = FALSE;
+	}
+
+	while ((c = str[i++]) != '\0') {
+		if (g_ascii_isdigit(c)) {
+			str[j++] = c;
+		}
+	}
+	str[j] = '\0';
+}
+
+/**
+ * Check if the version is a patched version.
+ * patched version are named like that x.y.z.patch where patch only contains digits.
+ * if patch contains alpha char it is because that's a RC or beta version. Not a patched one.
+ * @param version version to be tested
+ * @return 0 if the version is not patched. The version of the patch is returned otherwise.
+ */
+static guint check_for_patch(gchar *version, gboolean *is_rc, gboolean *is_beta) {
+	remove_alpha(version, is_rc, is_beta);
+	return (g_ascii_strtoull(version, NULL, 10));
+}
+
+version_number get_version_number_from_string(const gchar *input) {
+	version_number version = { 0 };
+	gchar **version_string = NULL;
+	const gchar *string = find_first_numeric(input);
+	if (!string)
+		goto the_end;
+	version_string = g_strsplit_set(string, ".-", -1);
+	version.major_version = g_ascii_strtoull(version_string[0], NULL, 10);
+	if (version_string[1])
+		version.minor_version = g_ascii_strtoull(version_string[1], NULL, 10);
+	else
+		goto the_end;
+	if (version_string[2])
+		version.micro_version = g_ascii_strtoull(version_string[2], NULL, 10);
+	else
+		goto the_end;
+	if (version_string[3] == NULL) {
+		version.patched_version = 0;
+		version.rc_version = FALSE;
+		version.beta_version = FALSE;
+	} else {
+		version.patched_version = check_for_patch(version_string[3], &version.rc_version, &version.beta_version);
+	}
+the_end:
+	g_strfreev(version_string);
+	return version;
+}
+
+version_number get_current_version_number() {
+	return get_version_number_from_string(PACKAGE_VERSION);
+}
+
+/**
+ * This function compares two version numbers following the pattern x.y.z[-type#]
+ * where type can be beta, rc, or a stable patch number.
+ * Version ordering: beta < rc < stable < stable-patch
+ * Examples: 1.4.0-beta1 < 1.4.0-rc2 < 1.4.0 < 1.4.0-1
+ * @param v1 First version number to be tested
+ * @param v2 Second version number to be tested
+ * @return -1 if v1 < v2, 1 if v1 > v2 and 0 if v1 is equal to v2
+ */
+int compare_version(version_number v1, version_number v2) {
+	if (v1.major_version < v2.major_version)
+		return -1;
+	else if (v1.major_version > v2.major_version)
+		return 1;
+	else {
+		if (v1.minor_version < v2.minor_version)
+			return -1;
+		else if (v1.minor_version > v2.minor_version)
+			return 1;
+		else {
+			if (v1.micro_version < v2.micro_version)
+				return -1;
+			else if (v1.micro_version > v2.micro_version)
+				return 1;
+			else {
+				// Determine version type
+				int v1_is_stable = !v1.rc_version && !v1.beta_version;
+				int v2_is_stable = !v2.rc_version && !v2.beta_version;
+
+				// Order: beta < rc < stable
+				if (v1.beta_version && !v2.beta_version) return -1;  // beta < (rc or stable)
+				if (v2.beta_version && !v1.beta_version) return 1;   // (rc or stable) > beta
+				if (v1.rc_version && v2_is_stable) return -1;        // rc < stable
+				if (v2.rc_version && v1_is_stable) return 1;         // stable > rc
+
+				// Same type: compare patched_version
+				if (v1.beta_version && v2.beta_version) {
+					// Both beta versions
+					if (v1.patched_version < v2.patched_version)
+						return -1;
+					else if (v1.patched_version > v2.patched_version)
+						return 1;
+				}
+				else if (v1.rc_version && v2.rc_version) {
+					// Both rc versions
+					if (v1.patched_version < v2.patched_version)
+						return -1;
+					else if (v1.patched_version > v2.patched_version)
+						return 1;
+				}
+				else if (v1_is_stable && v2_is_stable) {
+					// Both stable versions: compare patches
+					// 1.4.0 (patch=0) < 1.4.0-1 (patch=1)
+					if (v1.patched_version < v2.patched_version)
+						return -1;
+					else if (v1.patched_version > v2.patched_version)
+						return 1;
+				}
+			}
+		}
+	}
+	return 0;
+}
+
+// ============================================================================
+// NETWORK-DEPENDENT FUNCTIONS - Require libcurl
+// ============================================================================
+
+#if defined(HAVE_LIBCURL)
+
+static version_number get_last_version_number(gchar *version_str) {
+	gchar **v;
+	version_number version = { 0 };
+
+	v = g_strsplit_set(version_str, ".-", -1);
+
+	if (v[0])
+		version.major_version = g_ascii_strtoull(v[0], NULL, 10);
+	if (v[0] && v[1])
+		version.minor_version = g_ascii_strtoull(v[1], NULL, 10);
+	if (v[0] && v[1] && v[2])
+		version.micro_version = g_ascii_strtoull(v[2], NULL, 10);
+	if (v[0] && v[1] && v[2] && v[3]) {
+		remove_alpha(v[3], &version.rc_version, &version.beta_version);
+		version.patched_version = g_ascii_strtoull(v[3], NULL, 10);
+	}
+
+	g_strfreev(v);
+	return version;
+}
 
 static gboolean siril_update_get_highest(yyjson_doc *doc,
 		gchar **highest_version, gint64 *release_timestamp,
@@ -195,176 +366,39 @@ static gboolean siril_update_get_highest(yyjson_doc *doc,
 	return FALSE;
 }
 
-static void remove_alpha(gchar *str, gboolean *is_rc, gboolean *is_beta) {
-	unsigned long i = 0;
-	unsigned long j = 0;
-	char c;
-
-	if (g_str_has_prefix(str, "beta")) {
-		*is_rc = FALSE;
-		*is_beta = TRUE;
-	} else if (g_str_has_prefix(str, "rc")) {
-		*is_rc = TRUE;
-		*is_beta = FALSE;
-	} else {
-		*is_rc = FALSE;
-		*is_beta = FALSE;
-	}
-
-	while ((c = str[i++]) != '\0') {
-		if (g_ascii_isdigit(c)) {
-			str[j++] = c;
-		}
-	}
-	str[j] = '\0';
-}
-
-/**
- * Check if the version is a patched version.
- * patched version are named like that x.y.z.patch where patch only contains digits.
- * if patch contains alpha char it is because that's a RC or beta version. Not a patched one.
- * @param version version to be tested
- * @return 0 if the version is not patched. The version of the patch is returned otherwise.
- */
-static guint check_for_patch(gchar *version, gboolean *is_rc, gboolean *is_beta) {
-	remove_alpha(version, is_rc, is_beta);
-	return (g_ascii_strtoull(version, NULL, 10));
-}
-
-version_number get_version_number_from_string(const gchar *input) {
-	version_number version = { 0 };
-	gchar **version_string = NULL;
-	const gchar *string = find_first_numeric(input);
-	if (!string)
-		goto the_end;
-	version_string = g_strsplit_set(string, ".-", -1);
-	version.major_version = g_ascii_strtoull(version_string[0], NULL, 10);
-	if (version_string[1])
-		version.minor_version = g_ascii_strtoull(version_string[1], NULL, 10);
-	else
-		goto the_end;
-	if (version_string[2])
-		version.micro_version = g_ascii_strtoull(version_string[2], NULL, 10);
-	else
-		goto the_end;
-	if (version_string[3] == NULL) {
-		version.patched_version = 0;
-		version.rc_version = FALSE;
-		version.beta_version = FALSE;
-	} else {
-		version.patched_version = check_for_patch(version_string[3], &version.rc_version, &version.beta_version);
-	}
-the_end:
-	g_strfreev(version_string);
-	return version;
-}
-
-version_number get_current_version_number() {
-	return get_version_number_from_string(PACKAGE_VERSION);
-}
-
-static version_number get_last_version_number(gchar *version_str) {
-	gchar **v;
-	version_number version = { 0 };
-
-	v = g_strsplit_set(version_str, ".-", -1);
-
-	if (v[0])
-		version.major_version = g_ascii_strtoull(v[0], NULL, 10);
-	if (v[0] && v[1])
-		version.minor_version = g_ascii_strtoull(v[1], NULL, 10);
-	if (v[0] && v[1] && v[2])
-		version.micro_version = g_ascii_strtoull(v[2], NULL, 10);
-	if (v[0] && v[1] && v[2] && v[3]) {
-		remove_alpha(v[3], &version.rc_version, &version.beta_version);
-		version.patched_version = g_ascii_strtoull(v[3], NULL, 10);
-	}
-
-	g_strfreev(v);
-	return version;
-}
-
-/**
- * This function compares two version numbers following the pattern x.y.z[-type#]
- * where type can be beta, rc, or a stable patch number.
- * Version ordering: beta < rc < stable < stable-patch
- * Examples: 1.4.0-beta1 < 1.4.0-rc2 < 1.4.0 < 1.4.0-1
- * @param v1 First version number to be tested
- * @param v2 Second version number to be tested
- * @return -1 if v1 < v2, 1 if v1 > v2 and 0 if v1 is equal to v2
- */
-int compare_version(version_number v1, version_number v2) {
-	if (v1.major_version < v2.major_version)
-		return -1;
-	else if (v1.major_version > v2.major_version)
-		return 1;
-	else {
-		if (v1.minor_version < v2.minor_version)
-			return -1;
-		else if (v1.minor_version > v2.minor_version)
-			return 1;
-		else {
-			if (v1.micro_version < v2.micro_version)
-				return -1;
-			else if (v1.micro_version > v2.micro_version)
-				return 1;
-			else {
-				// Determine version type
-				int v1_is_stable = !v1.rc_version && !v1.beta_version;
-				int v2_is_stable = !v2.rc_version && !v2.beta_version;
-
-				// Order: beta < rc < stable
-				if (v1.beta_version && !v2.beta_version) return -1;  // beta < (rc or stable)
-				if (v2.beta_version && !v1.beta_version) return 1;   // (rc or stable) > beta
-				if (v1.rc_version && v2_is_stable) return -1;        // rc < stable
-				if (v2.rc_version && v1_is_stable) return 1;         // stable > rc
-
-				// Same type: compare patched_version
-				if (v1.beta_version && v2.beta_version) {
-					// Both beta versions
-					if (v1.patched_version < v2.patched_version)
-						return -1;
-					else if (v1.patched_version > v2.patched_version)
-						return 1;
-				}
-				else if (v1.rc_version && v2.rc_version) {
-					// Both rc versions
-					if (v1.patched_version < v2.patched_version)
-						return -1;
-					else if (v1.patched_version > v2.patched_version)
-						return 1;
-				}
-				else if (v1_is_stable && v2_is_stable) {
-					// Both stable versions: compare patches
-					// 1.4.0 (patch=0) < 1.4.0-1 (patch=1)
-					if (v1.patched_version < v2.patched_version)
-						return -1;
-					else if (v1.patched_version > v2.patched_version)
-						return 1;
-				}
-			}
-		}
-	}
-	return 0;
-}
-
 static gchar *parse_changelog(gchar *changelog) {
 	gchar **token;
 	GString *strResult;
-	guint nargs, i;
+	guint nargs;
 
 	token = g_strsplit(changelog, "\n", -1);
 	nargs = g_strv_length(token);
 
-	strResult = g_string_new(token[0]);
+	if (nargs < 2) {
+		g_strfreev(token);
+		return g_strdup("");
+	}
+
+	strResult = g_string_new(token[0]);  // Version (e.g., "siril-1.4.0")
+	strResult = g_string_append(strResult, "\n");
+	strResult = g_string_append(strResult, token[1]);  // Date (e.g., "12/05/25")
 	strResult = g_string_append(strResult, "\n\n");
-	/* we start at line 3 */
-	i = 3;
-	while (i < nargs && token[i][0] != '\0') {
+
+	/* Start reading from line 2 onwards */
+	int i = 2;
+	while (i < nargs) {
+		// Stop if we encounter another version line (starts with "siril")
+		if (g_str_has_prefix(g_strstrip(token[i]), "siril ") ||
+			g_str_has_prefix(g_strstrip(token[i]), "siril-")) {
+			break;
+			}
+
+		// Include all lines (even empty ones) to preserve formatting
 		strResult = g_string_append(strResult, token[i]);
 		strResult = g_string_append(strResult, "\n");
 		i++;
 	}
+
 	g_strfreev(token);
 	return g_string_free(strResult, FALSE);
 }
@@ -683,4 +717,152 @@ void siril_check_notifications(gboolean verbose) {
 	g_thread_new("siril-notifications", fetch_url_async, args);
 }
 
-#endif
+// Parse SPCC mirrors JSON and populate the global string vector
+static int parseJsonSpccMirrors(const char *jsonString) {
+	// Free existing catalogues if any
+	if (spcc_mirrors) {
+		g_strfreev(spcc_mirrors);
+		spcc_mirrors = NULL;
+	}
+
+	// Parse JSON from string using yyjson
+	yyjson_doc *doc = yyjson_read(jsonString, strlen(jsonString), YYJSON_READ_NOFLAG);
+	if (!doc) {
+		siril_log_color_message(_("Error parsing SPCC mirrors JSON: Failed to parse JSON\n"), "red");
+		return 1;
+	}
+
+	yyjson_val *root = yyjson_doc_get_root(doc);
+	if (!yyjson_is_arr(root)) {
+		siril_log_color_message(_("Error parsing SPCC mirrors JSON: Root is not an array\n"), "red");
+		yyjson_doc_free(doc);
+		return 1;
+	}
+
+	size_t length = yyjson_arr_size(root);
+	if (length == 0) {
+		siril_log_color_message(_("Error parsing SPCC mirrors JSON: Empty array\n"), "red");
+		yyjson_doc_free(doc);
+		return 1;
+	}
+
+	// Allocate string vector (NULL-terminated)
+	spcc_mirrors = g_new0(gchar *, length + 1);
+
+	// Iterate over mirror entries
+	size_t valid_count = 0;
+	for (size_t i = 0; i < length; i++) {
+		yyjson_val *mirror = yyjson_arr_get(root, i);
+		if (!yyjson_is_obj(mirror)) {
+			siril_log_color_message(_("Error parsing SPCC mirrors JSON: Entry is not a valid object\n"), "red");
+			continue;
+		}
+
+		// Get required fields
+		const char *url = yyjson_get_str(yyjson_obj_get(mirror, "url"));
+		const char *description = yyjson_get_str(yyjson_obj_get(mirror, "description"));
+
+		if (!url || !description) {
+			siril_log_color_message(_("Error parsing SPCC mirrors JSON: Required fields missing\n"), "red");
+			continue;
+		}
+
+		// Store the URL in the string vector
+		spcc_mirrors[valid_count] = g_strdup(url);
+		valid_count++;
+
+		siril_debug_print("SPCC mirror %zu: %s (%s)\n", valid_count, url, description);
+	}
+
+	// Clean up
+	yyjson_doc_free(doc);
+
+	if (valid_count == 0) {
+		g_strfreev(spcc_mirrors);
+		spcc_mirrors = NULL;
+		siril_log_color_message(_("Error parsing SPCC mirrors JSON: No valid mirrors found\n"), "red");
+		return 1;
+	}
+
+	// siril_log_message(_("Successfully loaded %zu SPCC mirror(s)\n"), valid_count);
+	return 0;
+}
+
+static gboolean end_spcc_mirrors_idle(gpointer p) {
+	fetch_url_async_data *args = (fetch_url_async_data *) p;
+	if (!args->content)
+		goto end_spcc_mirrors_error;
+
+	// Cache the JSON locally
+	gchar *spcc_mirror_path = g_build_path(G_DIR_SEPARATOR_S, siril_get_config_dir(),"siril", "spcc_mirrors.json", NULL);
+	GError *error = NULL;
+
+	if (g_file_set_contents(spcc_mirror_path, args->content, -1, &error)) {
+		siril_debug_print("Wrote spcc_mirrors.json file at %s\n", spcc_mirror_path);
+	} else {
+		// Handle error
+		siril_debug_print("Failed to write spcc_mirrors.json file: %s\n", error->message);
+		g_error_free(error);
+	}
+
+	// Parse JSON file and populate spcc_mirrors
+	if (parseJsonSpccMirrors(args->content) != 0) {
+		siril_log_message(_("Error fetching or parsing SPCC mirrors JSON file from URL\n"));
+		goto end_spcc_mirrors_error;
+	}
+	/* pre-check the Gaia archive status */
+	check_gaia_archive_status();
+
+
+end_spcc_mirrors_error:
+	set_cursor_waiting(FALSE);
+	/* free data */
+	free(args->content);
+	free(args);
+	set_progress_bar_data(PROGRESS_TEXT_RESET, PROGRESS_RESET);
+	stop_processing_thread();
+	return FALSE;
+}
+
+void siril_check_spcc_mirrors(gboolean verbose) {
+	// Try to read the local cached version of the SPCC mirrors file
+	gchar *spcc_mirror_path = g_build_path(G_DIR_SEPARATOR_S, siril_get_config_dir(), "siril", "spcc_mirrors.json", NULL);
+	if (g_file_test(spcc_mirror_path, G_FILE_TEST_EXISTS)) {
+		gchar *content = NULL;
+		gsize length;
+		GError *error = NULL;
+		if (g_file_get_contents(spcc_mirror_path, &content, &length, &error)) {
+			if (parseJsonSpccMirrors(content))
+				siril_debug_print("Failed to parse spcc_mirrors.json file\n");
+			else
+				siril_debug_print("Read spcc_mirrors.json file at %s\n", spcc_mirror_path);
+			g_free(content);
+		} else {
+			// Handle error
+			siril_debug_print("Failed to read spcc_mirrors.json: %s", error->message);
+			g_error_free(error);
+		}
+	}
+	g_free(spcc_mirror_path);
+
+	if (!is_online()) {
+		siril_log_color_message(_("Siril is in offline mode, cannot check SPCC mirrors.\n"), "red");
+		return;
+	}
+
+	fetch_url_async_data *args = calloc(1, sizeof(fetch_url_async_data));
+	GString *url = g_string_new(GITLAB_URL);
+	g_string_append_printf(url, "/%s/%s", BRANCH, SPCC_MIRRORS);
+	args->url = g_string_free(url, FALSE);
+	siril_debug_print("SPCC mirrors URL: %s\n", args->url);
+	args->content = NULL;
+	args->verbose = verbose;
+	args->idle_function = end_spcc_mirrors_idle;
+
+	siril_log_message(_("Checking SPCC mirrors...\n"));
+
+	// this is a graphical operation, we don't use the main processing thread for it, it could block file opening
+	g_thread_new("siril-spcc-mirrors", fetch_url_async, args);
+}
+
+#endif // HAVE_LIBCURL
