@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2025 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,18 +20,15 @@
 #ifndef SRC_CORE_SIRIL_UPDATE_H_
 #define SRC_CORE_SIRIL_UPDATE_H_
 
-#if defined(HAVE_JSON_GLIB) && defined(HAVE_NETWORKING)
+int compare_version(version_number v1, version_number v2);
+version_number get_version_number_from_string(const gchar *string);
+version_number get_current_version_number();
 
-typedef struct {
-	guint major_version;
-	guint minor_version;
-	guint micro_version;
-	guint patched_version;
-	gboolean beta_version;
-	gboolean rc_version;
-} version_number;
+#if ( defined(HAVE_LIBCURL)  || defined(HAVE_LIBGIT2) )
 
 void siril_check_updates(gboolean verbose);
+void siril_check_spcc_mirrors(gboolean verbose);
+void siril_check_notifications(gboolean verbose);
 
 #endif
 
