@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2025 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/icc_profile.h"
 #include "core/processing.h"
 #include "core/siril_log.h"
 #include "core/siril_date.h"
@@ -31,10 +32,10 @@
 #include "gui/utils.h"
 #include "gui/message_dialog.h"
 #include "gui/dialog_preview.h"
-#include "gui/utils.h"
+#include "gui/dialogs.h"
+#include "gui/icc_profile.h"
 #include "gui/image_display.h"
 #include "gui/callbacks.h"
-#include "gui/dialogs.h"
 #include "io/Astro-TIFF.h"
 #include "io/conversion.h"
 #include "io/sequence.h"
@@ -46,21 +47,6 @@
 static image_type type_of_image = TYPEUNDEF;
 static SirilWidget *saveDialog = NULL;
 
-static void gtk_filter_add(GtkFileChooser *file_chooser, const gchar *title,
-		const gchar *pattern, gboolean set_default) {
-	GtkFileFilter *f = gtk_file_filter_new();
-	gtk_file_filter_set_name(f, title);
-	/* get the patterns */
-	gchar **patterns = g_strsplit(pattern, ";", -1);
-	for (gint i = 0; patterns[i] != NULL; i++)
-		gtk_file_filter_add_pattern(f, patterns[i]);
-	/* free the patterns */
-	g_strfreev(patterns);
-	gtk_file_chooser_add_filter(file_chooser, f);
-	if (set_default)
-		gtk_file_chooser_set_filter(file_chooser, f);
-}
-
 static void set_filters_save_dialog(GtkFileChooser *chooser) {
 	GString *all_filter = NULL;
 	const gchar *fits_filter = "*.fit;*.FIT;*.fits;*.FITS;*.fts;*.FTS;*.fit.fz;*.FIT.fz;*.fits.fz;*.FITS.fz;*.fts.fz;*.FTS.fz";
@@ -86,6 +72,14 @@ static void set_filters_save_dialog(GtkFileChooser *chooser) {
 	all_filter = g_string_append(all_filter, jpg_filter);
 #endif
 
+#ifdef HAVE_LIBJXL
+	const gchar *jxl_filter = "*.jxl;*.JXL";
+
+	gtk_filter_add(chooser, _("JPEG XL Files (*.jxl)"), jxl_filter, FALSE);
+	all_filter = g_string_append(all_filter, ";");
+	all_filter = g_string_append(all_filter, jxl_filter);
+#endif
+
 #ifdef HAVE_LIBPNG
 	const gchar *png_filter = "*.png;*.PNG";
 
@@ -133,6 +127,9 @@ static image_type get_filetype(const gchar *filter) {
 		} else if (!g_strcmp0(string[i], "tif")) {
 			type = TYPETIFF;
 			break;
+		} else if (!g_strcmp0(string[i], "jxl")) {
+			type = TYPEJXL;
+			break;
 		} else if (!g_strcmp0(string[i], "ppm")) {
 			type = TYPEPNM;
 			break;
@@ -217,12 +214,14 @@ static void prepare_savepopup() {
 	static GtkNotebook* notebookFormat = NULL;
 	static GtkWidget *savepopup = NULL;
 	static GtkWidget *savetxt = NULL;
+	static GtkWidget *button_savepopup = NULL;
 	int tab;
 
 	if (notebookFormat == NULL) {
 		notebookFormat = GTK_NOTEBOOK(lookup_widget("notebookFormat"));
 		savepopup = lookup_widget("savepopup");
 		savetxt = lookup_widget("filenameframe");
+		button_savepopup = lookup_widget("button_savepopup");
 	}
 
 	GtkWindow *parent = GTK_WINDOW(GTK_APPLICATION_WINDOW(lookup_widget("control_window")));
@@ -244,10 +243,15 @@ static void prepare_savepopup() {
 		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving JPG"));
 		tab = PAGE_JPG;
 		break;
+	case TYPEJXL:
+		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving JPEG XL"));
+		tab = PAGE_JXL;
+		break;
 	case TYPETIFF:
 		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving TIFF"));
 		set_copyright_in_TIFF();
 		set_description_in_TIFF();
+		set_icc_description_in_TIFF();
 		tab = PAGE_TIFF;
 		break;
 	default:
@@ -260,6 +264,7 @@ static void prepare_savepopup() {
 
 	gtk_widget_set_visible(savetxt, FALSE);
 	gtk_notebook_set_current_page(notebookFormat, tab);
+	gtk_widget_grab_focus(button_savepopup);
 }
 
 static void init_dialog() {
@@ -337,6 +342,11 @@ static void filter_changed(gpointer user_data) {
 		new_filename = g_strdup_printf("%s.jpg", file_no_ext);
 		break;
 #endif
+#ifdef HAVE_LIBJXL
+	case TYPEJXL:
+		new_filename = g_strdup_printf("%s.jxl", file_no_ext);
+		break;
+#endif
 #ifdef HAVE_LIBPNG
 	case TYPEPNG:
 		new_filename = g_strdup_printf("%s.png", file_no_ext);
@@ -458,10 +468,10 @@ static gboolean end_save(gpointer p) {
 	gtk_entry_set_text(args->entry, "");
 	gtk_widget_hide(lookup_widget("savepopup"));
 	display_filename(); // update filename display
-	set_precision_switch();
+	gui_function(set_precision_switch, NULL);
 	set_cursor_waiting(FALSE);
 	close_dialog();
-	update_MenuItem();
+	gui_function(update_MenuItem, NULL);
 
 	g_free(args->copyright);
 	g_free(args->description);
@@ -479,22 +489,31 @@ static gboolean test_for_viewer_mode() {
 	return confirm;
 }
 
-static gboolean initialize_data(gpointer p) {
+static void initialize_data(gpointer p) {
 	struct savedial_data *args = (struct savedial_data *) p;
 
 	GtkToggleButton *fits_8 = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton_save_fit8"));
 	GtkToggleButton *fits_16s = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton_save_fit16s"));
 	GtkToggleButton *fits_16 = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton_save_fit16"));
 	GtkToggleButton *update_hilo = (GTK_TOGGLE_BUTTON(lookup_widget("checkbutton_update_hilo")));
+	GtkToggleButton *checksum = (GTK_TOGGLE_BUTTON(lookup_widget("checkbutton_chksum")));
 #ifdef HAVE_LIBJPEG
 	GtkSpinButton *qlty_spin_button = GTK_SPIN_BUTTON(lookup_widget("quality_spinbutton"));
 	args->quality = gtk_spin_button_get_value_as_int(qlty_spin_button);
 #endif
+#ifdef HAVE_LIBJXL
+	GtkSpinButton *quality_spin_button = GTK_SPIN_BUTTON(lookup_widget("jxl_quality_spinbutton"));
+	args->jxl_quality = gtk_spin_button_get_value_as_int(quality_spin_button);
+	GtkSpinButton *effort_spin_button = GTK_SPIN_BUTTON(lookup_widget("jxl_effort_spinbutton"));
+	args->jxl_effort = gtk_spin_button_get_value_as_int(effort_spin_button);
+	GtkToggleButton *toggle_button_8bit = GTK_TOGGLE_BUTTON(lookup_widget("jxl_force_8bit"));
+	args->jxl_force_8bit = gtk_toggle_button_get_active(toggle_button_8bit);
+#endif
 #ifdef HAVE_LIBTIFF
 	GtkToggleButton *button_8 = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton8bits"));
 	GtkToggleButton *button_32 = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton32bits"));
 	args->bitspersamples = gtk_toggle_button_get_active(button_8) ? 8 : gtk_toggle_button_get_active(button_32) ? 32 : 16;
-	get_tif_data_from_ui(&gfit, &args->description, &args->copyright, &args->embeded_icc);
+	get_tif_data_from_ui(&gfit, &args->description, &args->copyright);
 	args->tiff_compression = get_tiff_compression();
 #endif
 	args->entry = GTK_ENTRY(lookup_widget("savetxt"));
@@ -510,10 +529,82 @@ static gboolean initialize_data(gpointer p) {
 		args->bitpix = FLOAT_IMG;
 
 	args->update_hilo = gtk_toggle_button_get_active(update_hilo);
-
-	return test_for_viewer_mode();
+	args->checksum = gtk_toggle_button_get_active(checksum);
 }
 
+#ifdef HAVE_JPEG
+static long calculate_jpeg_size(struct savedial_data *args) {
+	char const *tmp_dir = g_get_tmp_dir();
+	gchar *tmp_filename;
+	gchar *tmp_filename_template = (com.uniq->filename != NULL) ? g_path_get_basename(com.uniq->filename) : g_strdup(_("preview"));
+/* Due to a Windows behavior and Mingw temp file name,
+ * we escapes the special characters by inserting a '\' before them */
+#ifdef _WIN32
+	gchar *tmp = g_build_filename(tmp_dir, tmp_filename_template, NULL);
+	tmp_filename = g_strescape(tmp, NULL);
+	g_free(tmp);
+#else
+    tmp_filename = g_build_filename(tmp_dir, tmp_filename_template, NULL);
+#endif
+	g_free(tmp_filename_template);
+	if (!g_str_has_suffix(tmp_filename, ".jpg") && (!g_str_has_suffix(tmp_filename, ".jpeg"))) {
+		tmp_filename = str_append(&tmp_filename, ".jpg");
+	}
+	if (g_file_test(tmp_filename, G_FILE_TEST_EXISTS)) {
+		if (g_unlink(tmp_filename)) {
+			siril_debug_print("Failed to remove existing temporary file");
+			g_free(tmp_filename);
+			return -1;
+		}
+	}
+	// Save JPEG to temporary file
+	if (savejpg(tmp_filename, &gfit, args->quality, FALSE)) {
+		siril_debug_print("Failed to save JPEG to temporary file");
+		g_free(tmp_filename);
+		return -1;
+	}
+	// Get file size
+	GStatBuf stat_buf;
+	goffset file_size = 0L;
+	if (g_stat(tmp_filename, &stat_buf) == 0) {
+		file_size = stat_buf.st_size;
+	} else {
+		g_warning("Unable to get file size for '%s': %s", tmp_filename, g_strerror(errno));
+	}
+	if (g_unlink(tmp_filename)) {
+		siril_debug_print("g_unlink() failed\n");
+		g_free(tmp_filename);
+	}
+	return (long) file_size;
+}
+
+static gboolean end_calculate_jpeg_size(gpointer p) {
+	struct savedial_data *args = (struct savedial_data *) p;
+	stop_processing_thread();
+	set_cursor_waiting(FALSE);
+	g_free(args->copyright);
+	g_free(args->description);
+	free(args);
+	return FALSE;
+}
+
+static gpointer calculate_jpeg_size_thread(gpointer p) {
+	struct savedial_data *args = (struct savedial_data *) p;
+	long file_size = calculate_jpeg_size(args);
+	gchar *txt = g_format_size_full(file_size, G_FORMAT_SIZE_IEC_UNITS);
+	gtk_entry_set_text(GTK_ENTRY(lookup_widget("size_estimate_entry")), txt);
+	siril_add_idle(end_calculate_jpeg_size, args);
+	return NULL;
+}
+
+#else
+static gpointer calculate_jpeg_size_thread(gpointer p) {
+	printf("Should not happen\n");
+	return NULL;
+}
+
+#endif
+
 static gpointer mini_save_dialog(gpointer p) {
 	struct savedial_data *args = (struct savedial_data *) p;
 #ifdef HAVE_LIBPNG
@@ -527,13 +618,18 @@ static gpointer mini_save_dialog(gpointer p) {
 			args->retval = savebmp(args->filename, &gfit);
 			break;
 #ifdef HAVE_LIBJPEG
-		case TYPEJPG:
-			args->retval = savejpg(args->filename, &gfit, args->quality);
+		case TYPEJPG:;
+			args->retval = savejpg(args->filename, &gfit, args->quality, TRUE);
+			break;
+#endif
+#ifdef HAVE_LIBJXL
+		case TYPEJXL:
+			args->retval = savejxl(args->filename, &gfit, args->jxl_effort, args->jxl_quality, args->jxl_force_8bit);
 			break;
 #endif
 #ifdef HAVE_LIBTIFF
 		case TYPETIFF:
-			args->retval = savetif(args->filename, &gfit, args->bitspersamples, args->description, args->copyright, args->tiff_compression, args->embeded_icc, TRUE);
+			args->retval = savetif(args->filename, &gfit, args->bitspersamples, args->description, args->copyright, args->tiff_compression, TRUE, TRUE);
 			break;
 #endif
 #ifdef HAVE_LIBPNG
@@ -550,28 +646,36 @@ static gpointer mini_save_dialog(gpointer p) {
 			/* Check if MIPS-HI and MIPS-LO must be updated. If yes,
 			 * Values are taken from the layer 0 */
 			if (args->update_hilo) {
-				gfit.hi = gui.hi;
-				gfit.lo = gui.lo;
+				gfit.keywords.hi = gui.hi;
+				gfit.keywords.lo = gui.lo;
 				if (gfit.orig_bitpix == BYTE_IMG
-						&& (gfit.hi > UCHAR_MAX || gfit.lo > UCHAR_MAX)) {
-					gfit.hi = UCHAR_MAX;
-					gfit.lo = 0;
+						&& (gfit.keywords.hi > UCHAR_MAX || gfit.keywords.lo > UCHAR_MAX)) {
+					gfit.keywords.hi = UCHAR_MAX;
+					gfit.keywords.lo = 0;
 				} else if (gfit.orig_bitpix == SHORT_IMG
-						&& (gfit.hi > SHRT_MAX || gfit.lo > SHRT_MAX)) {
-					gfit.hi = UCHAR_MAX;
-					gfit.lo = 0;
+						&& (gfit.keywords.hi > SHRT_MAX || gfit.keywords.lo > SHRT_MAX)) {
+					gfit.keywords.hi = UCHAR_MAX;
+					gfit.keywords.lo = 0;
 				}
 				if (gfit.orig_bitpix == BYTE_IMG && gfit.bitpix != BYTE_IMG) {
-					gfit.hi = USHRT_MAX;
-					gfit.lo = 0;
+					gfit.keywords.hi = USHRT_MAX;
+					gfit.keywords.lo = 0;
 				}
 			} else {
-				gfit.hi = 0;
-				gfit.lo = 0;
+				gfit.keywords.hi = 0;
+				gfit.keywords.lo = 0;
 			}
+			gfit.checksum = args->checksum;
 			args->retval = savefits(args->filename, &gfit);
 			if (!args->retval && single_image_is_loaded()) {
 				com.uniq->filename = strdup(args->filename);
+				// com.uniq->filename is handled by libc functions so we can't use g_strdup_printf directly
+				if (!g_str_has_suffix(args->filename, com.pref.ext)) {
+					gchar* tempfilename = g_strdup_printf("%s%s", args->filename, com.pref.ext);
+					free(com.uniq->filename);  // Free the previously allocated memory
+					com.uniq->filename = strdup(tempfilename);
+					g_free(tempfilename);  // Use g_free for GLib allocated memory
+				}
 				com.uniq->fileexist = TRUE;
 			}
 			break;
@@ -604,12 +708,66 @@ void on_savetxt_changed(GtkEditable *editable, gpointer user_data) {
 	gtk_widget_set_sensitive(button, (*name != '\0'));
 }
 
+void on_size_estimate_toggle_toggled(GtkToggleButton *button, gpointer user_data) {
+	struct savedial_data *args = calloc(1, sizeof(struct savedial_data));
+	if (!args) {
+		PRINT_ALLOC_ERR;
+		return;
+	}
+	initialize_data(args);
+
+	if (!get_thread_run() && gtk_toggle_button_get_active(button)) {
+		if (!get_thread_run()) {
+			if (!start_in_new_thread(calculate_jpeg_size_thread, args)) {
+				g_free(args->copyright);
+				g_free(args->description);
+				free(args);
+			}
+			return;
+		}
+	}
+	// Clear preview size
+	gtk_entry_set_text(GTK_ENTRY(lookup_widget("size_estimate_entry")), "");
+
+	g_free(args->description);
+	g_free(args->copyright);
+	free (args);
+}
+
+void on_quality_spinbutton_value_changed(GtkSpinButton *button, gpointer user_data) {
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("size_estimate_toggle")))) {
+		struct savedial_data *args = calloc(1, sizeof(struct savedial_data));
+		if (!args) {
+			PRINT_ALLOC_ERR;
+			return;
+		}
+		initialize_data(args);
+		if (!get_thread_run()) {
+			if (!start_in_new_thread(calculate_jpeg_size_thread, args)) {
+				g_free(args->copyright);
+				g_free(args->description);
+				free(args);
+			}
+		} else {
+			g_free(args->description);
+			g_free(args->copyright);
+			free (args);
+			return;
+		}
+	}
+}
+
 void on_button_savepopup_clicked(GtkButton *button, gpointer user_data) {
 	struct savedial_data *args = calloc(1, sizeof(struct savedial_data));
 
 	set_cursor_waiting(TRUE);
-	if (initialize_data(args)) {
-		start_in_new_thread(mini_save_dialog, args);
+	initialize_data(args);
+	if (test_for_viewer_mode()) {
+		if (!start_in_new_thread(mini_save_dialog, args)) {
+			g_free(args->copyright);
+			g_free(args->description);
+			free(args);
+		}
 	} else {
 		g_free(args->copyright);
 		g_free(args->description);
@@ -622,8 +780,13 @@ void on_savetxt_activate(GtkEntry *entry, gpointer user_data) {
 	struct savedial_data *args = calloc(1, sizeof(struct savedial_data));
 
 	set_cursor_waiting(TRUE);
-	if (initialize_data(args)) {
-		start_in_new_thread(mini_save_dialog, args);
+	initialize_data(args);
+	if (test_for_viewer_mode()) {
+		if (!start_in_new_thread(mini_save_dialog, args)) {
+			g_free(args->copyright);
+			g_free(args->description);
+			free(args);
+		}
 	} else {
 		g_free(args->copyright);
 		g_free(args->description);
@@ -646,8 +809,13 @@ void on_header_save_as_button_clicked() {
 				struct savedial_data *args = calloc(1, sizeof(struct savedial_data));
 
 				set_cursor_waiting(TRUE);
-				if (initialize_data(args)) {
-					start_in_new_thread(mini_save_dialog, args);
+				initialize_data(args);
+				if (test_for_viewer_mode()) {
+					if (!start_in_new_thread(mini_save_dialog, args)) {
+						g_free(args->description);
+						g_free(args->copyright);
+						free(args);
+					}
 				} else {
 					g_free(args->copyright);
 					g_free(args->description);
@@ -741,8 +909,9 @@ void on_header_snapshot_button_clicked(gboolean clipboard) {
 		if (clipboard) {
 			GtkClipboard *cb = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
 			gtk_clipboard_set_image(cb, pixbuf);
+#if !defined _WIN32
 			gtk_clipboard_store(cb);
-
+#endif
 			GtkWidget *w = lookup_widget("header_snapshot_button");
 			GtkWidget *popover = snapshot_notification(w, NULL, pixbuf);
 			g_timeout_add(5000, (GSourceFunc) snapshot_notification_close, (gpointer) popover);
